/// <reference types="node" />
import * as Logger from '@nexus/logger';
import * as NexusSchema from '@nexus/schema';
import { CreateFieldResolverInfo } from '@nexus/schema/dist/core';
import { GraphQLFieldResolver, GraphQLResolveInfo } from 'graphql';
import * as HTTP from 'http';
import { NexusSchemaStatefulBuilders } from '../../lib/nexus-schema-stateful';
import { RuntimeContributions } from '../../lib/plugin';
import { MaybePromise } from '../../lib/utils';
import { AppState } from '../app';
import { SettingsData, SettingsInput } from './settings';
export interface Request extends HTTP.IncomingMessage {
    log: Logger.Logger;
}
export declare type ContextContributor<Req> = (req: Req) => MaybePromise<Record<string, unknown>>;
declare type MiddlewareFn = (source: any, args: any, context: NexusSchema.core.GetGen<'context'>, info: GraphQLResolveInfo, next: GraphQLFieldResolver<any, any>) => any;
export interface Schema extends NexusSchemaStatefulBuilders {
    /**
     * todo link to website docs
     */
    use: (schemaPlugin: NexusSchema.core.NexusPlugin) => void;
    /**
     * todo link to website docs
     */
    middleware: (fn: (config: CreateFieldResolverInfo) => MiddlewareFn | undefined) => void;
    /**
     * todo link to website docs
     */
    addToContext: <Req = Request>(contextContributor: ContextContributor<Req>) => void;
}
interface SchemaInternal {
    private: {
        state: {
            settings: SettingsData;
            schemaPlugins: NexusSchema.core.NexusPlugin[];
            contextContributors: ContextContributor<any>[];
        };
        /**
         * Create the Nexus GraphQL Schema
         */
        makeSchema: (plugins: RuntimeContributions[]) => {
            /**
             * GraphQL schema built by @nexus/schema
             */
            schema: NexusSchema.core.NexusGraphQLSchema;
            /**
             * Assert that there are no missing types after running typegen only
             * so that we don't block writing the typegen when eg: renaming types
             */
            assertValidSchema: () => void;
        };
        settings: {
            data: SettingsData;
            change: (newSettings: SettingsInput) => void;
        };
    };
    public: Schema;
}
export declare function create(appState: AppState): SchemaInternal;
export {};
//# sourceMappingURL=schema.d.ts.map