"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const NexusSchema = tslib_1.__importStar(require("@nexus/schema"));
const core_1 = require("@nexus/schema/dist/core");
const common_tags_1 = require("common-tags");
const Layout = tslib_1.__importStar(require("../../lib/layout"));
const nexus_schema_stateful_1 = require("../../lib/nexus-schema-stateful");
const logger_1 = require("./logger");
const settings_1 = require("./settings");
function create(appState) {
    const statefulNexusSchema = nexus_schema_stateful_1.createNexusSchemaStateful();
    const state = {
        schemaPlugins: [],
        settings: {},
        contextContributors: [],
    };
    const middleware = (fn) => {
        api.public.use(NexusSchema.plugin({
            // TODO: Do we need to expose the name property?
            name: 'local-middleware',
            onCreateFieldResolver(config) {
                return fn(config);
            },
        }));
    };
    const api = {
        public: Object.assign(Object.assign({}, statefulNexusSchema.builders), { use(plugin) {
                if (appState.isWasServerStartCalled === true) {
                    logger_1.log.warn(common_tags_1.stripIndent `
            A Nexus Schema plugin was ignored because it was loaded after the server was started
            Make sure to call \`schema.use\` before you call \`server.start\`
          `);
                }
                state.schemaPlugins.push(plugin);
            },
            addToContext(contextContributor) {
                state.contextContributors.push(contextContributor);
            },
            middleware }),
        private: {
            state: state,
            makeSchema: (plugins) => {
                const nexusSchemaConfig = settings_1.mapSettingsToNexusSchemaConfig(plugins, state.settings);
                nexusSchemaConfig.types.push(...statefulNexusSchema.state.types);
                nexusSchemaConfig.plugins.push(...state.schemaPlugins);
                const { schema, missingTypes } = core_1.makeSchemaInternal(nexusSchemaConfig);
                return {
                    schema,
                    assertValidSchema() {
                        NexusSchema.core.assertNoMissingTypes(schema, missingTypes);
                        if (statefulNexusSchema.state.types.length === 0) {
                            logger_1.log.warn(Layout.schema.emptyExceptionMessage());
                        }
                    },
                };
            },
            settings: {
                data: state.settings,
                change(newSettings) {
                    settings_1.changeSettings(state.settings, newSettings);
                },
            },
        },
    };
    return api;
}
exports.create = create;
//# sourceMappingURL=schema.js.map